@@ -1,9 +1,9 @@
 /*
- * $Id: bulletin_gui.c,v 1.26 2004/08/19 19:38:14 we7u Exp $
+ * $Id: bulletin_gui.c,v 1.37 2006/04/24 19:21:41 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,8 +22,10 @@
  * Look at the README for more information on the program.
  */
 
-
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -32,7 +34,12 @@
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
+
+// Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
+
 #include <ctype.h>
 #include <sys/types.h>
 
@@ -69,6 +76,9 @@
 #include "interface.h"
 #include "util.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 Widget Display_bulletins_dialog = NULL;
@@ -144,7 +154,7 @@ void bulletin_message(char *call_sign, char *tag, char *packet_message, time_t s
     distance = distance_from_my_station(call_sign,temp_my_course);
     xastir_snprintf(temp, sizeof(temp), "%-9s:%-4s (%s %6.1f %s) %s\n",
             call_sign, &tag[3], time_str, distance,
-            units_english_metric ? langcode("UNIOP00004"): langcode("UNIOP00005"),
+            english_units ? langcode("UNIOP00004"): langcode("UNIOP00005"),
             packet_message);
 
 // Operands of <= have incompatible types (double, int):
@@ -237,7 +247,7 @@ void bulletin_data_add(char *call_sign, char *from_call, char *data,
     (void)msg_data_add(call_sign,
         from_call,
         data,
-        "",
+        " ",    // Need something here.  Empty string no good.
         MESSAGE_BULLETIN,
         from,
         &temp_bulletin_record);
@@ -246,13 +256,17 @@ void bulletin_data_add(char *call_sign, char *from_call, char *data,
     if (temp_bulletin_record == -1L) {
         char temp[10];
 
+
+//fprintf(stderr,"We think it's a new bulletin!\n");
+ 
         // We add to the distance in order to come up with 0.0
         // if the distance is not known at all (no position
         // found yet).
         distance = (int)(distance_from_my_station(from_call,temp) + 0.9999);
 
         if ( (bulletin_range == 0)
-                || (distance <= bulletin_range && distance > 0) ) {
+                || (distance <= bulletin_range && distance > 0)
+                || (view_zero_distance_bulletins && distance == 0.0) ) {
             // We have a _new_ bulletin that's within our
             // current range setting.  Note that it's also possible
             // to have a zero distance for the bulletin (we haven't
@@ -273,6 +287,7 @@ void bulletin_data_add(char *call_sign, char *from_call, char *data,
             }
 
             if (pop_up_new_bulletins) {
+//fprintf(stderr,"bulletin_data_add: popping up bulletins\n");
                 popup_bulletins();
                 if (debug_level & 1)
                     fprintf(stderr,"\n");
@@ -398,7 +413,15 @@ static void zero_bulletin_processing(Message *fill) {
                             fprintf(stderr,"Filled in distance for earlier bulletin:%d miles\n",
                                 distance);
                         }
-                        popup_bulletins();
+
+                        // If view_zero_distance_bulletins was not
+                        // turned on, then we probably haven't seen
+                        // this bulletin until now.  Popup up the
+                        // Bulletin dialog.
+                        if (!view_zero_distance_bulletins) {
+//fprintf(stderr,"zero_bulletin_processing: popping up bulletins\n");
+                            popup_bulletins();
+                        }
                     }
                 }
             }
@@ -431,14 +454,14 @@ static void find_zero_position_bulletins(void) {
 // that are newer than first_new_bulletin_time and fit within our
 // range.  If any found, it updates the Bulletins dialog.
 time_t last_bulletin_check = (time_t)0l;
-void check_for_new_bulletins() {
 
+void check_for_new_bulletins(int curr_sec) {
 
     // Check every 15 seconds max
-    if ( (last_bulletin_check + 15) > sec_now() ) {
+    if ( (last_bulletin_check + 15) > curr_sec ) {
         return;
     }
-    last_bulletin_check = sec_now();
+    last_bulletin_check = curr_sec;
 
     // Look first to see if we might be able to fill in positions on
     // any older bulletins, then cause a popup for those that fit
@@ -454,7 +477,7 @@ void check_for_new_bulletins() {
 
     // Enough time passed since most recent bulletin?  Need to have
     // enough time to perhaps fill in a distance for each bulletin.
-    if ( (last_new_bulletin_time + 15) > sec_now() ) {
+    if ( (last_new_bulletin_time + 15) > curr_sec ) {
         //fprintf(stderr,"Not enough time has passed\n");
         return;
     }
@@ -474,6 +497,7 @@ void check_for_new_bulletins() {
 //fprintf(stderr,"%d new bulletins found\n",new_bulletin_count);
 
     if (new_bulletin_count) {
+//fprintf(stderr,"check_for_new_bulletins: popping up bulletins\n");
         popup_bulletins();
 
         if (debug_level & 1)
@@ -559,7 +583,7 @@ void  Zero_Bulletin_Data_toggle( /*@unused@*/ Widget widget, XtPointer clientDat
 void Bulletins(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*@unused@*/ XtPointer callData) {
     Widget pane, form, button_range, button_close, dist, dist_units;
     unsigned int n;
-    Arg args[20];
+    Arg args[50];
     Atom delw;
     char temp[10];
 
@@ -572,7 +596,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
 
         Display_bulletins_dialog = XtVaCreatePopupShell(langcode("BULMW00001"),
                 xmDialogShellWidgetClass,
-                Global.top,
+                appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
@@ -628,7 +652,7 @@ begin_critical_section(&display_bulletins_dialog_lock, "bulletin_gui.c:Bulletins
                 XmNnavigationType, XmTAB_GROUP,
                 NULL);
 
-        dist_units = XtVaCreateManagedWidget((units_english_metric?langcode("UNIOP00004"):langcode("UNIOP00005")),
+        dist_units = XtVaCreateManagedWidget((english_units?langcode("UNIOP00004"):langcode("UNIOP00005")),
                 xmLabelWidgetClass, 
                 form,
                 XmNtopAttachment, XmATTACH_FORM,
