@@ -1,9 +1,9 @@
 /*
- * $Id: bulletin_gui.h,v 1.9 2005/01/08 09:46:48 we7u Exp $
+ * $Id: bulletin_gui.h,v 1.11 2006/01/17 21:05:20 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,7 +31,7 @@ extern int bulletin_range;
 // From bulletin_gui.c
 extern void bulletin_gui_init(void);
 extern void bulletin_data_add(char *call_sign, char *from_call, char *data, char *seq, char type, char from);
-extern void check_for_new_bulletins(void);
+extern void check_for_new_bulletins(int current_sec);
 
 #endif  // __XASTIR_BULLETIN_GUI_H
 
