@@ -1,15 +1,19 @@
 
-// $Id: geo-find.c,v 1.8 2005/01/08 09:54:35 we7u Exp $
+// $Id: geo-find.c,v 1.12 2006/04/24 19:21:41 we7u Exp $
 
 
 /* Copyright 2002 Daniel Egnor.  See LICENSE.geocoder file.
- * Portions Copyright (C) 2004-2005  The Xastir Group
+ * Portions Copyright (C) 2004-2006  The Xastir Group
  *
  * The geo_find() function defined here uses an address map built by 
  * geo-*-to-* to parse an address and convert it to geographical
  * coordinates.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "geo.h"
 #include "io.h"
 
@@ -18,6 +22,11 @@
 #include <ctype.h>
 #include <stdio.h>
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 #define D(x)
 
 
@@ -307,7 +316,7 @@ static const char *input_word(struct state *s,const char *pos) {
 	}
 
 	while (s->input_begin != s->input_end 
-	   && (s->input_depth > 0 || !isalnum(*s->input_begin))) {
+	   && (s->input_depth > 0 || !isalnum((int)*s->input_begin))) {
 		const char ch = *s->input_begin++;
 		if ('(' == ch)
 			++s->input_depth;
@@ -317,9 +326,9 @@ static const char *input_word(struct state *s,const char *pos) {
 
 	while (s->input_begin != s->input_end
 	   &&  s->buffer_end != &s->buffer[sizeof s->buffer]
-           &&  isalnum(*s->input_begin)) {
+           &&  isalnum((int)*s->input_begin)) {
 		if (s->buffer == s->buffer_end
-		|| !isdigit(s->buffer_end[-1]) || !isalpha(*s->input_begin))
+		|| !isdigit((int)s->buffer_end[-1]) || !isalpha((int)*s->input_begin))
 			*s->buffer_end++ = *s->input_begin;
 		++s->input_begin;
 	}
