@@ -1,9 +1,9 @@
 /*
- * $Id: geocoder_gui.c,v 1.15 2005/07/21 20:04:40 we7u Exp $
+ * $Id: geocoder_gui.c,v 1.21 2006/04/24 19:21:41 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -37,10 +40,15 @@
 #include "xastir.h"
 #include "main.h"
 #include "lang.h"
-#include "maps.h"
+//#include "maps.h"
 #include "io.h"
 #include "geo.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 Widget geocoder_place_dialog = (Widget)NULL;
 Widget geocoder_zip_data = (Widget)NULL;
 Widget geocoder_state_data = (Widget)NULL;
@@ -266,8 +274,9 @@ void Geocoder_place(/*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*
 
 begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_place" );
 
+        // Find Address
         geocoder_place_dialog = XtVaCreatePopupShell(langcode("PULDNMP029"),
-                xmDialogShellWidgetClass,Global.top,
+                xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
@@ -283,8 +292,8 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
                 XmNshadowThickness, 1,
                 NULL);
 
-//        address = XtVaCreateManagedWidget(langcode("FEATURE004"),xmLabelWidgetClass, form,
-        address = XtVaCreateManagedWidget("Address:",xmLabelWidgetClass, form,
+        // Address:
+        address = XtVaCreateManagedWidget(langcode("FEATURE007"),xmLabelWidgetClass, form,
                 XmNtopAttachment, XmATTACH_FORM,
                 XmNtopOffset, 10,
                 XmNbottomAttachment, XmATTACH_NONE,
@@ -315,9 +324,8 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
                 XmNtraversalOn, TRUE,
                 NULL);
 
-//        locality = XtVaCreateManagedWidget(langcode("FEATURE003"),xmLabelWidgetClass, form,
-        locality = XtVaCreateManagedWidget("City:",xmLabelWidgetClass, form,
- 
+        // City:
+        locality = XtVaCreateManagedWidget(langcode("FEATURE008"),xmLabelWidgetClass, form,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, address,
                 XmNtopOffset, 10,
@@ -350,6 +358,7 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
                 XmNtraversalOn, TRUE,
                 NULL);
 
+        // State/Province:
         state = XtVaCreateManagedWidget(langcode("FEATURE002"),xmLabelWidgetClass, form,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, locality,
@@ -383,8 +392,8 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
                 XmNtraversalOn, TRUE,
                 NULL);
 
-//        show_dest_toggle = XtVaCreateManagedWidget(langcode("FEATURE002"),xmToggleButtonGadgetClass, form,
-        show_dest_toggle = XtVaCreateManagedWidget("Mark Destination",xmToggleButtonGadgetClass, form,
+        // Mark Destination
+        show_dest_toggle = XtVaCreateManagedWidget(langcode("FEATURE009"),xmToggleButtonGadgetClass, form,
                 XmNvisibleWhenOff, TRUE,
                 XmNindicatorSize, 12,
                 XmNtopAttachment, XmATTACH_WIDGET,
@@ -405,8 +414,8 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
             XmToggleButtonSetState(show_dest_toggle,TRUE,FALSE);
 
 
-//        zip = XtVaCreateManagedWidget(langcode("FEATURE001"),xmLabelWidgetClass, form,
-        zip = XtVaCreateManagedWidget("Zip Code:",xmLabelWidgetClass, form,
+        // Zip Code:
+        zip = XtVaCreateManagedWidget(langcode("FEATURE010"),xmLabelWidgetClass, form,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, state,
                 XmNtopOffset, 10,
@@ -438,9 +447,8 @@ begin_critical_section(&geocoder_place_dialog_lock, "geocoder_gui.c:Geocoder_pla
                 XmNtraversalOn, TRUE,
                 NULL);
 
-//        map_file = XtVaCreateManagedWidget(langcode("FEATURE006"),xmLabelWidgetClass, form,
-        map_file = XtVaCreateManagedWidget("Geocoding File",xmLabelWidgetClass, form,
- 
+        // Geocoding File:
+        map_file = XtVaCreateManagedWidget(langcode("FEATURE011"),xmLabelWidgetClass, form,
                 XmNtopAttachment, XmATTACH_WIDGET,
                 XmNtopWidget, zip,
                 XmNtopOffset, 10,
