@@ -1,9 +1,9 @@
 /*
- * $Id: hostname.c,v 1.18 2005/07/06 15:22:43 we7u Exp $
+ * $Id: hostname.c,v 1.26 2006/04/24 19:21:41 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <Xm/XmAll.h>
@@ -46,13 +49,25 @@
 #include "main.h"
 #include "lang.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+#ifndef HAVE_SIGHANDLER_T
+  #ifdef HAVE_SIG_T
+    typedef sig_t sighandler_t;
+  #else
+    typedef void (*sighandler_t)(int);
+  #endif
+#endif
+
 
 #ifndef __LCLINT__
-  #ifndef sigjmp_buf
+  #ifndef HAVE_SIGJMP_BUF
     jmp_buf ret_place;
-  #else // sigjmp_buf
+  #else // HAVE_SIGJMP_BUF
     static  sigjmp_buf ret_place;       /* Jump address if alarm */
-  #endif    // sigjmp_buf
+  #endif    // HAVE_SIGJMP_BUF
 #endif // __LCLINT__
 
 
@@ -138,6 +153,11 @@ char *host_lookup(char *host, char *ip, int ip_size, int time) {
             if (host_pid==0) {  // We're in the child process
 
 
+                // Go back to default signal handler instead of
+                // calling restart() on SIGHUP
+                (void) signal(SIGHUP,SIG_DFL);
+
+
                 // Change the name of the new child process.  So far
                 // this only works for "ps" listings, not for "top".
                 // This code only works on Linux.  For BSD use
@@ -159,7 +179,11 @@ char *host_lookup(char *host, char *ip, int ip_size, int time) {
                 if (debug_level & 1024)
                     fprintf(stderr,"Set alarm \n");
 
+#ifdef RETSIGTYPE
+                previous_loc = (RETSIGTYPE *)signal(SIGALRM, host_time_out);
+#else
                 previous_loc = signal(SIGALRM, host_time_out);
+#endif
 
                 // Set up to jump here if we time out on SIGALRM
                 if (sigsetjmp(ret_place,-1)!=0) {
@@ -168,7 +192,7 @@ char *host_lookup(char *host, char *ip, int ip_size, int time) {
                     (void)alarm(0);
 
                     // Reset the SIGALRM handler to its previous value
-                    (void)signal(SIGALRM, previous_loc);
+                    (void)signal(SIGALRM, (sighandler_t)previous_loc);
 
                     // Return net connection time out
                     xastir_snprintf(ip_addr, sizeof(ip_addr), "TIMEOUT");
@@ -192,7 +216,7 @@ char *host_lookup(char *host, char *ip, int ip_size, int time) {
                 // Turn off the alarm
                 (void)alarm(0);
                 // Reset the SIGALRM handler to its previous value
-                (void)signal(SIGALRM, previous_loc);
+                (void)signal(SIGALRM, (sighandler_t)previous_loc);
 
                 if (hostinfo) {
                     names = hostinfo -> h_aliases;
