@@ -1,9 +1,9 @@
 /*
- * $Id: interface.h,v 1.32 2005/06/14 19:59:24 we7u Exp $
+ * $Id: interface.h,v 1.36 2006/02/13 20:30:56 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,6 @@
 #define __XASTIR_INTERFACE_H
 
 #include <termios.h>
-#include <pthread.h>
 #include "util.h"
 #include "xastir.h"
 
@@ -224,11 +223,14 @@ extern void update_interface_list(void);
 extern int WX_rain_gauge_type;
 
 /* interface.c */
+extern int is_local_interface(int port);
+extern int is_network_interface(int port);
 extern void send_agwpe_packet(int xastir_interface, int RadioPort, unsigned char type, unsigned char *FromCall, unsigned char *ToCall, unsigned char *Path, unsigned char *Data, int length);
 
 extern unsigned char *incoming_data;
 extern int incoming_data_length;
 extern unsigned char incoming_data_copy[MAX_LINE_SIZE];
+extern unsigned char incoming_data_copy_previous[MAX_LINE_SIZE];
 extern int NETWORK_WAITTIME;
 extern void startup_all_or_defined_port(int port);
 extern void shutdown_all_active_or_defined_port(int port);
