@@ -1,14 +1,25 @@
 
-// $Id: io-common.c,v 1.2 2005/01/08 09:54:35 we7u Exp $
+// $Id: io-common.c,v 1.6 2006/04/24 19:21:41 we7u Exp $
 
 
 /* Copyright 2002 Daniel Egnor.  See LICENSE.geocoder file.
- * Portions Copyright (C) 2004-2005  The Xastir Group
+ * Portions Copyright (C) 2004-2006  The Xastir Group
  */
-#include "io.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif  // HAVE_CONFIG_H
+
+#include "io.h"
 #include <ctype.h>
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
+
+
 int io_out_i4(struct io_file *f,int pos,int o) {
         unsigned char x[4];
         x[0] = o >> 24;
@@ -51,9 +62,9 @@ int io_strntoi(const char *str,int len) {
 	int r = 0;
 	int sign = 1;
 	const char * const end = str + len;
-	while (end != str && isspace(*str)) ++str;
+	while (end != str && isspace((int)*str)) ++str;
 	if (end != str && *str == '-') (sign *= -1),++str;
 	if (end != str && *str == '+') ++str;
-	while (end != str && isdigit(*str)) r = *str++ - '0' + 10 * r;
+	while (end != str && isdigit((int)*str)) r = *str++ - '0' + 10 * r;
 	return r * sign;
 }
