@@ -1,9 +1,9 @@
 /*
- * $Id: location_gui.c,v 1.15 2005/07/31 06:19:12 we7u Exp $
+ * $Id: location_gui.c,v 1.18 2006/04/24 19:21:41 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,9 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,6 +39,10 @@
 #include "lang.h"
 #include "snprintf.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 Widget location_dialog = (Widget)NULL;
 Widget location_list;
