@@ -1,9 +1,9 @@
 /*
- * $Id: main.h,v 1.81 2005/07/20 05:09:52 we7u Exp $
+ * $Id: main.h,v 1.92 2006/02/14 19:32:12 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,10 +34,24 @@
 #endif  // __linux__
 
 
-// Define this if your pthreads implementation uses SIGUSR1 signal.
-// This will disable our SIGUSR1 handler which allows us to create
-// snapshots on receipt of that signal.
-//#define OLD_PTHREADS
+// This gets defined if pthreads implementation uses SIGUSR1/SIGUSR2
+// signals.  This disables our SIGUSR1 handler which disallows
+// creating snapshots on receipt of that signal.  Old kernels (2.0
+// and early 2.1) had only 32 signals available so only USR1/USR2
+// were available for LinuxThreads use.  Newer kernels have more
+// signals available, making USR1/USR2 available to programs again.
+// _NSIG is defined in /usr/include/bits/signum.h
+//
+#ifdef __linux__
+# ifdef _NSIG
+#  if (_NSIG <= 32)
+#   define OLD_PTHREADS
+//#   warning ***** OLD_PTHREADS DETECTED *****
+#  else
+//#   warning ***** NEW_PTHREADS DETECTED *****
+#  endif // if (_NSIG...)
+# endif  // _NSIG
+#endif  // __linux__
 
 
 // To enable the "Transmit Raw WX data" button in
@@ -48,6 +62,10 @@
 //
 //#define TRANSMIT_RAW_WX
 
+// To use predefined object configuration files from within the 
+// user's base directory e.g. ~/.xastir/config/predefined_SAR.sys
+// rather than from the main base directory, enable this definition.
+//#define OBJECT_DEF_FILE_USER_BASE
 
 extern int enable_server_port;
 
@@ -82,13 +100,6 @@ extern int interrupt_drawing_now;
 #define MAX_PHG      8
 #define MAX_COMMENT  80
 
-#define MAX_UNPROTO  100
-
-#define MAX_TNC_CHARS 128
-#define MAX_NET_CHARS 128
-#define MAX_GPS_CHARS 128
-#define MAX_WX_CHARS 4
-
 
 
 //////////////////////////////////////////////////////////////////////
@@ -108,10 +119,13 @@ extern void init_set_proc_title(int argc, char *argv[], char *envp[]);
 
 // New stuff defined by Xastir project:
 extern int my_argc;
-extern void *my_argv;
-extern void *my_envp;
+extern char **my_argv;
+extern char **my_envp;
 //////////////////////////////////////////////////////////////////////
 
+extern int input_x;
+extern int input_y;
+
 #define MAX_RELAY_DIGIPEATER_CALLS 50
 extern char relay_digipeater_calls[10*MAX_RELAY_DIGIPEATER_CALLS];
 extern int skip_dupe_checking;
@@ -165,9 +179,14 @@ extern int sb_high_speed_limit;
 extern int pop_up_new_bulletins;
 extern int view_zero_distance_bulletins;
 extern int warn_about_mouse_modifiers;
+extern int predefined_menu_from_file;
+
+extern int draw_labeled_grid_border;  // used to turn on or off border on map
 
 extern int output_station_type;
 
+extern int emergency_beacon;
+
 typedef struct _selections {
     int none;
     int mine;
@@ -227,8 +246,8 @@ extern What_to_display Display_;
 extern int currently_selected_stations;
 extern int currently_selected_stations_save;
 extern int colors[256];
-extern int max_trail_colors;
-extern int trail_colors[32];
+#define MAX_TRAIL_COLORS 32
+extern int trail_colors[MAX_TRAIL_COLORS];
 extern int current_trail_color;
 extern int english_units;
 extern int do_dbstatus;
@@ -321,6 +340,18 @@ extern int coordinate_system;
 #define USE_UTM_SPECIAL 4
 #define USE_MGRS        5
 
+typedef struct {
+    Widget calling_dialog;  // NULL if the calling dialog has been closed.
+    Widget input_lat_deg;   // Pointers to calling dialog's widgets
+    Widget input_lat_min;   // (Where to get/put the data)
+    Widget input_lat_dir;
+    Widget input_lon_deg;
+    Widget input_lon_min;
+    Widget input_lon_dir;
+} coordinate_calc_array_type;
+extern coordinate_calc_array_type coordinate_calc_array;
+extern void Coordinate_calc(Widget w, XtPointer clientData, XtPointer callData);
+
 
 extern void HandlePendingEvents(XtAppContext app);
 extern void create_gc(Widget w);
@@ -375,7 +406,6 @@ extern void updateObjectPictureCallback(Widget w,XtPointer clientData,XtPointer
 
 extern void Draw_All_CAD_Objects(Widget w);
 
-
 // unit conversion
 extern char un_alt[2+1];
 extern char un_dst[2+1];
