@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_geo.c,v 1.56 2005/07/26 19:17:00 tvrusso Exp $
+ * $Id: map_geo.c,v 1.65 2006/04/24 19:21:41 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,10 @@
 
 //#define FUZZYRASTER
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -37,7 +40,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -126,6 +131,9 @@
 #undef XASTIR_PACKAGE_VERSION
 #endif // HAVE_IMAGEMAGICK
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 extern int npoints;		/* tsk tsk tsk -- globals */
@@ -553,10 +561,12 @@ void draw_geo_image_map (Widget w,
 #ifdef TIMING_DEBUG
     time_mark(1);
 #endif  // TIMING_DEBUG
-#ifdef USE_MAP_CACHE 
-    int map_cache_return;
-#endif  // USE_MAP_CACHE
 
+#ifdef HAVE_IMAGEMAGICK
+  #ifdef USE_MAP_CACHE 
+    int map_cache_return;
+  #endif  // USE_MAP_CACHE
+#endif  // HAVE_IMAGEMAGICK
 
 
     xastir_snprintf(file, sizeof(file), "%s/%s", dir, filenm);
@@ -1238,7 +1248,16 @@ fprintf(stderr,"1 ");
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1282,7 +1301,16 @@ fprintf(stderr,"1 ");
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1417,7 +1445,16 @@ fprintf(stderr,"1 ");
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1451,7 +1488,16 @@ fprintf(stderr,"2 ");
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1485,7 +1531,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1528,7 +1583,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1545,7 +1609,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1562,7 +1635,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1579,7 +1661,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1596,7 +1687,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1613,7 +1713,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1632,7 +1741,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1667,7 +1785,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1725,7 +1852,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1746,7 +1882,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1767,7 +1912,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1815,11 +1969,14 @@ fprintf(stderr,"2 ");
 //fprintf(stderr,"Checking for transparency\n");
 
                 // Get the color allocated on < 8bpp displays. pixel color is written to my_colors.pixel
-                if (visual_type == NOT_TRUE_NOR_DIRECT)
+                if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                    XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                     XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-                else
+                }
+                else {
                     pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                     &my_colors[l].pixel);
+                }
 
                 if (check_trans(my_colors[l],trans_color) ) {
 
@@ -1840,11 +1997,14 @@ fprintf(stderr,"2 ");
  
 
             // Get the color allocated on < 8bpp displays. pixel color is written to my_colors.pixel
-            if (visual_type == NOT_TRUE_NOR_DIRECT)
+            if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                 XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-            else
+            }
+            else {
                 pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                 &my_colors[l].pixel);
+            }
 
             if (debug_level & 16)
                 fprintf(stderr,"Color allocated is %li  %i  %i  %i \n", my_colors[l].pixel,
@@ -1859,7 +2019,16 @@ fprintf(stderr,"2 ");
         if (image_info)
             DestroyImageInfo(image_info);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1909,7 +2078,16 @@ fprintf(stderr,"2 ");
     HandlePendingEvents(app_context);
     if (interrupt_drawing_now) {
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -1953,7 +2131,16 @@ fprintf(stderr,"2 ");
         if (xi)
             XDestroyImage (xi);
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -2053,7 +2240,16 @@ fprintf(stderr,"2 ");
             XDestroyImage (xi);
 #endif  // HAVE_IMAGEMAGICK
         // Update to screen
-        (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+        (void)XCopyArea(XtDisplay(da),
+            pixmap,
+            XtWindow(da),
+            gc,
+            0,
+            0,
+            (unsigned int)screen_width,
+            (unsigned int)screen_height,
+            0,
+            0);
         return;
     }
 
@@ -2063,6 +2259,31 @@ fprintf(stderr,"2 ");
                 (map_y_0 <= map_y_max) || (map_proj == 1 && !map_done && scr_y < screen_height);
                 map_y_0++, c_y += map_c_dy) {
 
+        HandlePendingEvents(app_context);
+        if (interrupt_drawing_now) {
+#ifdef HAVE_IMAGEMAGICK
+            if (image)
+                DestroyImage(image);
+            if (image_info)
+                DestroyImageInfo(image_info);
+#else   // HAVE_IMAGEMAGICK
+            if (xi)
+                XDestroyImage (xi);
+#endif // HAVE_IMAGEMAGICK
+            // Update to screen
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
+            return;
+        }
+
         scr_y = (c_y - y_lat_offset) / scale_y;
         if (scr_y != scr_yp) {                  // don't do a row twice
             scr_yp = scr_y;                     // remember as previous y
