@@ -1,9 +1,9 @@
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil -*-
- * $Id: map_tiger.c,v 1.41 2005/07/22 06:08:22 we7u Exp $
+ * $Id: map_tiger.c,v 1.49 2006/04/24 19:21:42 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,10 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -36,7 +39,9 @@
 #include <errno.h>
 
 // Needed for Solaris
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif  // HAVE_STRINGS_H
 
 #include <dirent.h>
 #include <netinet/in.h>
@@ -113,6 +118,9 @@
 #undef XASTIR_PACKAGE_VERSION
 #endif // HAVE_IMAGEMAGICK
 
+// Must be last include file
+#include "leak_detection.h"
+
 
 
 extern int npoints;		/* tsk tsk tsk -- globals */
@@ -494,7 +502,16 @@ void draw_tiger_map (Widget w,
         HandlePendingEvents(app_context);
         if (interrupt_drawing_now) {
             // Update to screen
-            (void)XCopyArea(XtDisplay(da),pixmap,XtWindow(da),gc,0,0,screen_width,screen_height,0,0);
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
             return;
         }
 
@@ -653,11 +670,14 @@ void draw_tiger_map (Widget w,
             }
 
             // Get the color allocated on < 8bpp displays. pixel color is written to my_colors.pixel
-            if (visual_type == NOT_TRUE_NOR_DIRECT)
+            if (visual_type == NOT_TRUE_NOR_DIRECT) {
+//                XFreeColors(XtDisplay(w), cmap, &(my_colors[l].pixel),1,0);
                 XAllocColor(XtDisplay(w), cmap, &my_colors[l]);
-            else
+            }
+            else {
                 pack_pixel_bits(my_colors[l].red, my_colors[l].green, my_colors[l].blue,
                                 &my_colors[l].pixel);
+            }
 
             if (debug_level & 512)
                 fprintf(stderr,"Color allocated is %li  %i  %i  %i \n", my_colors[l].pixel,
@@ -814,6 +834,26 @@ void draw_tiger_map (Widget w,
     // loop over map pixel rows
     for (map_y_0 = map_y_min, c_y = (double)c_y_min; (map_y_0 <= map_y_max); map_y_0++, c_y += map_c_dy) {
 
+        HandlePendingEvents(app_context);
+        if (interrupt_drawing_now) {
+            if (image)
+               DestroyImage(image);
+            if (image_info)
+               DestroyImageInfo(image_info);
+            // Update to screen
+            (void)XCopyArea(XtDisplay(da),
+                pixmap,
+                XtWindow(da),
+                gc,
+                0,
+                0,
+                (unsigned int)screen_width,
+                (unsigned int)screen_height,
+                0,
+                0);
+            return;
+        }
+
         scr_y = (c_y - y_lat_offset) / scale_y;
         if (scr_y != scr_yp) {                  // don't do a row twice
             scr_yp = scr_y;                     // remember as previous y
