@@ -1,9 +1,9 @@
 /*
- * $Id: messages.c,v 1.56 2005/07/06 15:22:47 we7u Exp $
+ * $Id: messages.c,v 1.64 2006/04/24 19:21:42 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <Xm/XmAll.h>
@@ -61,6 +64,10 @@
 #include "util.h"
 #include "interface.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 char group_data_file[400];
 char *group_data_list = NULL;   // Need this NULL for Solaris!
@@ -745,7 +752,7 @@ void output_message(char *from, char *to, char *message, char *path) {
 // this lock.
 end_critical_section(&send_message_dialog_lock, "db.c:update_messages" );
 
-                msg_data_add(to,
+                (void)msg_data_add(to,
                     from,
                     message_out,
                     message_pool[i].seq,
@@ -776,6 +783,16 @@ begin_critical_section(&send_message_dialog_lock, "db.c:update_messages" );
 
 
 
+// Here we're doing some routing of the transmitted packets.  We
+// want to keep Xastir from transmitting on ports that aren't
+// actively being used in the QSO, but also cover the case where
+// ports can go up/down during the QSO.
+//
+// Note that igates might get into the act quite a bit for RF<->RF
+// QSO's if we're sending to the internet too, but that's a bug in
+// the igate software, and not something that Xastir should try to
+// correct itself.
+//
 void transmit_message_data(char *to, char *message, char *path) {
     DataRow *p_station;
 
@@ -783,51 +800,125 @@ void transmit_message_data(char *to, char *message, char *path) {
         fprintf(stderr,"Transmitting data to %s : %s\n",to,message);
 
     p_station = NULL;
-    if (search_station_name(&p_station,to,1)) {
+
+
+    if (strcmp(to, my_callsign) == 0) { // My station message
+
+        // Send out all active ports
+
         if (debug_level & 2)
-            fprintf(stderr,"found station %s\n",p_station->call_sign);
+            fprintf(stderr,"My call VIA any way\n");
 
-        if (strcmp(to,my_callsign)!=0) {
-            /* check to see if it was heard via a TNC otherwise send it to the last port heard */
-            if ((p_station->flag & ST_VIATNC) != 0) {        // heard via TNC ?
-                output_my_data(message,p_station->heard_via_tnc_port,0,0,0,path);
-                /* station heard via tnc but in the past hour? */
-                /* if not heard with in the hour try via net */
-                if (!heard_via_tnc_in_past_hour(to)) {
-                    if (p_station->data_via==DATA_VIA_NET) {
-                        /* try last port heard */
-                        output_my_data(message,p_station->last_port_heard,0,0,0,path);
-                    }
-                }
-            } else {
-                /* if not a TNC then a NET port? */
-                if (p_station->data_via==DATA_VIA_NET) {
-                    /* try last port herd */
-                    output_my_data(message,p_station->last_port_heard,0,0,0,path);
-                } else {
-                    /* Not a TNC or a NET try all possible */
-                    if (debug_level & 2)
-                        fprintf(stderr,"VIA any way\n");
+        output_my_data(message,-1,0,0,0,path);
 
-                    output_my_data(message,-1,0,0,0,path);
-                }
-            }
-        } else {
-            /* my station message */
-            /* try all possible */
-            if (debug_level & 2)
-                fprintf(stderr,"My call VIA any way\n");
+        // All done 
+        return;
+    }
+
+
+    if (!search_station_name(&p_station,to,1)) {
+
+        // No data record found for this station.  Send to all
+        // active ports.
 
-            output_my_data(message,-1,0,0,0,path);
-        }
-    } else {
-        /* no data found try every way*/
-        /* try all possible */
         if (debug_level & 2)
             fprintf(stderr,"VIA any way\n");
 
         output_my_data(message,-1,0,0,0,path);
+
+        // All done
+        return;
     }
+
+
+    if (debug_level & 2)
+        fprintf(stderr,"found station %s\n",p_station->call_sign);
+
+
+    // It's not being sent to my callsign but to somebody else
+    // "out there".  Because the truth is...
+
+
+    if ( ((p_station->flag & ST_VIATNC) != 0)
+            && (heard_via_tnc_in_past_hour(to)) ) {
+
+        int port_num;
+
+
+        // Station was heard via a TNC port within the previous
+        // hour.  Send to TNC port it was heard on.
+        // 
+        output_my_data(message,p_station->heard_via_tnc_port,0,0,0,path);
+
+        // Send to all internet ports.  Iterate through the port
+        // definitions looking for internet ports, send the message
+        // out once to each.
+        //
+        for (port_num = 0; port_num < MAX_IFACE_DEVICES; port_num++) {
+
+            // If it's an internet port, send the message.
+            if (port_data[port_num].device_type == DEVICE_NET_STREAM) {
+                output_my_data(message,port_num,0,0,0,path);
+            }
+        }
+
+        // All done
+        return;
+    }
+
+    else if (p_station->data_via==DATA_VIA_NET) {
+        int port_num;
+        int active_internet_ports_found = 0;
+
+
+        // Station was heard over an internet interface.  Check
+        // whether we have any internet interfaces available with TX
+        // enabled.  If so, send out those ports.  Else drop through
+        // and hit the TRANSMIT-ALL clause at the end of this
+        // function.
+
+        // Iterate through the port definitions looking for internet
+        // ports with transmit enabled.  Send the message out once
+        // to each.
+        //
+        for (port_num = 0; port_num < MAX_IFACE_DEVICES; port_num++) {
+
+            // If it's an internet port and transmit is enabled,
+            // send the message and set the flag.
+            if ( (port_data[port_num].device_type == DEVICE_NET_STREAM)
+                    && (port_data[port_num].active == DEVICE_IN_USE)
+                    && (port_data[port_num].status == DEVICE_UP)
+                    && (devices[port_num].transmit_data == 1) ) {
+
+                // Found a tx-enabled internet port that was up and
+                // running.  Send the message out this port.
+                output_my_data(message,port_num,0,0,0,path);
+
+                active_internet_ports_found++;
+            }
+        }
+
+        if (active_internet_ports_found) {
+            // We found at least one tx-enabled internet interface
+            // that was up and running.
+
+            // All done.
+            return;
+        }
+        else {  // No active tx-enabled internet ports were found.
+                // Drop through to the TRANSMIT-ALL clause below.
+        } 
+    }
+
+
+    // We've NOT heard this station on a TNC port within the
+    // last hour and have no active tx-enabled internet ports to
+    // send to.  Send to ALL active ports.
+
+    if (debug_level & 2)
+        fprintf(stderr,"VIA any way\n");
+
+    output_my_data(message,-1,0,0,0,path);
 }
 
 
@@ -934,16 +1025,16 @@ void transmit_message_data_delayed(char *to, char *message,
 time_t delayed_transmit_last_check = (time_t)0;
 
 
-void check_delayed_transmit_queue(void) {
+void check_delayed_transmit_queue(int curr_sec) {
     delayed_ack_record_p ptr = delayed_ack_list_head;
     int active_records = 0;
 
 
     // Skip this function if we did it during this second already.
-    if (delayed_transmit_last_check == sec_now()) {
+    if (delayed_transmit_last_check == curr_sec) {
         return;
     }
-    delayed_transmit_last_check = sec_now();
+    delayed_transmit_last_check = curr_sec;
 
 //fprintf(stderr, "Checking delayed TX queue for something to transmit.\n");
 //fprintf(stderr, ".");
@@ -1033,10 +1124,10 @@ void check_and_transmit_messages(time_t time) {
 
 
     // Skip this function if we did it during this second already.
-    if (last_check_and_transmit == sec_now()) {
+    if (last_check_and_transmit == time) {
         return;
     }
-    last_check_and_transmit = sec_now();
+    last_check_and_transmit = time;
 
     for (i=0; i<MAX_OUTGOING_MESSAGES;i++) {
         if (message_pool[i].active==MESSAGE_ACTIVE) {
@@ -1221,15 +1312,30 @@ fprintf(stderr,
 
 // Function which marks a message as ack'ed in the transmit queue
 // and releases the next message to allow it to be transmitted.
+// Handles REPLY-ACK format or normal ACK format just fine.
+//
 void clear_acked_message(char *from, char *to, char *seq) {
     int i,ii;
     int found;
     char lowest[3];
     char temp1[MAX_CALLSIGN+1];
     char *temp_ptr;
+    char msg_id[5+1];
 
 
-    (void)remove_trailing_spaces(seq);  // This is IMPORTANT here!!!
+    // Copy seq into local variable
+    xastir_snprintf(msg_id,
+        sizeof(msg_id),
+        "%s",
+        seq);
+
+    // Check for REPLY-ACK protocol.  If found, terminate at the end
+    // of the first ack.
+    temp_ptr = strchr(msg_id, '}');
+    if (temp_ptr)
+        *temp_ptr = '\0';
+
+    (void)remove_trailing_spaces(msg_id);  // This is IMPORTANT here!!!
 
     //lowest=100000;
     // Highest Base-90 2-char string
@@ -1244,7 +1350,8 @@ void clear_acked_message(char *from, char *to, char *seq) {
                     to,
                     message_pool[i].to_call_sign,
                     from,
-                    message_pool[i].from_call_sign,seq,
+                    message_pool[i].from_call_sign,
+                    msg_id,
                     message_pool[i].seq);
 
             if (strcmp(message_pool[i].to_call_sign,from)==0) {
@@ -1253,7 +1360,7 @@ void clear_acked_message(char *from, char *to, char *seq) {
                 if (strcmp(message_pool[i].from_call_sign,to)==0) {
                     if (debug_level & 1)
                         fprintf(stderr,"Matched message from_call_sign\n");
-                    if (strcmp(message_pool[i].seq,seq)==0) {
+                    if (strcmp(message_pool[i].seq,msg_id)==0) {
                         if (debug_level & 2)
                             fprintf(stderr,"Found and cleared\n");
 
@@ -1312,7 +1419,7 @@ end_critical_section(&send_message_dialog_lock, "messages.c:clear_acked_message"
                     }
                     else {
                         if (debug_level & 1)
-                            fprintf(stderr,"Sequences didn't match: %s %s\n",message_pool[i].seq,seq);
+                            fprintf(stderr,"Sequences didn't match: %s %s\n",message_pool[i].seq,msg_id);
                     }
                 }
             }
