@@ -1,9 +1,9 @@
 /*
- * $Id: messages.h,v 1.16 2005/01/08 09:46:48 we7u Exp $
+ * $Id: messages.h,v 1.18 2006/01/17 21:05:20 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -83,7 +83,7 @@ extern char group_data_file[400];
 extern void clear_acked_message(char *from, char *to, char *seq);
 extern void transmit_message_data(char *to, char *message, char *path);
 extern void transmit_message_data_delayed(char *to, char *message, char *path, time_t when);
-extern void check_delayed_transmit_queue(void);
+extern void check_delayed_transmit_queue(int curr_sec);
 
 
 //extern void output_message(char *from, char *to, char *message);
