@@ -1,9 +1,9 @@
 /*
- * $Id: messages_gui.c,v 1.43 2005/07/31 06:19:14 we7u Exp $
+ * $Id: messages_gui.c,v 1.49 2006/04/24 19:21:42 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdlib.h>
@@ -34,6 +37,10 @@
 #include "main.h"
 #include "lang.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 #define MAX_PATH 200
 
@@ -275,7 +282,9 @@ void get_send_message_path(char *callsign, char *path, int path_size) {
                 temp_ptr);
             XtFree(temp_ptr);
 
+            (void)remove_trailing_dash_zero(temp1);
             (void)to_upper(temp1);
+
             if(strcmp(temp1,my_callsign)==0) {
                 found = ii;
                 break;
@@ -344,6 +353,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_d
 
         (void)remove_leading_spaces(temp1);
         (void)remove_trailing_spaces(temp1);
+        (void)remove_trailing_dash_zero(temp1);
 
         if(temp1[0] != '\0') {
             // Yep, we have a custom path.  Warn the user that
@@ -406,6 +416,8 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Check_new_call
             temp_ptr);
         XtFree(temp_ptr);
 
+        (void)remove_trailing_dash_zero(call_sign);
+
         // Try lowercase
         get_path_data(call_sign, path, MAX_PATH);
         if (strlen(path) == 0) {
@@ -448,7 +460,6 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
 
     if (mw[ii].send_message_dialog) {
 
-        /* find station and go there */
         temp_ptr = XmTextFieldGetString(mw[ii].send_message_call_data);
         xastir_snprintf(temp1,
             sizeof(temp1),
@@ -458,6 +469,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
 
         (void)remove_trailing_spaces(temp1);
         (void)to_upper(temp1);
+        (void)remove_trailing_dash_zero(temp1);
 
         temp_ptr = XmTextFieldGetString(mw[ii].send_message_message_data);
         xastir_snprintf(temp2,
@@ -469,7 +481,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message_n
         // We have the message text now.  Check it for illegal
         // characters, remove them and substitute spaces if found.
         // Illegal characters are '|', '{', and '~' for messaging.
-        for (jj = 0; jj < strlen(temp2); jj++) {
+        for (jj = 0; jj < (int)strlen(temp2); jj++) {
             if (       temp2[jj] == '|'
                     || temp2[jj] == '{'
                     || temp2[jj] == '~' ) {
@@ -541,7 +553,6 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Clear_message_
 
     if (mw[i].send_message_dialog) {
 
-        /* find station and go there */
         temp_ptr = XmTextFieldGetString(mw[i].send_message_call_data);
         xastir_snprintf(temp1,
             sizeof(temp1),
@@ -551,6 +562,8 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Clear_message_
 
         (void)remove_trailing_spaces(temp1);  
         (void)to_upper(temp1);
+        (void)remove_trailing_dash_zero(temp1);
+
         /*fprintf(stderr,"Clear message from <%s> to <%s>\n",temp1,my_callsign);*/
         mdelete_messages_from(temp1);
         new_message_data=1;
@@ -575,7 +588,7 @@ void Clear_message_to( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/
 begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Clear_message_to" );
 
     if (mw[i].send_message_dialog) {
-        /* find station and go there */
+
         temp_ptr = XmTextFieldGetString(mw[i].send_message_call_data);
         xastir_snprintf(temp1,
             sizeof(temp1),
@@ -585,6 +598,8 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Clear_message_
 
         (void)remove_trailing_spaces(temp1);  
         (void)to_upper(temp1);
+        (void)remove_trailing_dash_zero(temp1);
+
         /*fprintf(stderr,"Clear message to <%s>\n",temp1);*/
         mdelete_messages_to(temp1);
         new_message_data=1;
@@ -644,12 +659,14 @@ void Kick_timer( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*/ XtPoi
 
 
     temp_ptr = XmTextFieldGetString(mw[atoi(clientData)].send_message_call_data);
-        xastir_snprintf(temp1,
-            sizeof(temp1),
-            "%s",
-            temp_ptr);
+    xastir_snprintf(temp1,
+        sizeof(temp1),
+        "%s",
+        temp_ptr);
     XtFree(temp_ptr);
 
+    (void)remove_trailing_dash_zero(temp1);
+
     kick_outgoing_timer(temp1);
 }
 
@@ -669,6 +686,8 @@ void Clear_messages_to( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*
             temp_ptr);
     XtFree(temp_ptr);
 
+    (void)remove_trailing_dash_zero(temp1);
+
     clear_outgoing_messages_to(temp1);
     update_messages(1); // Force an update to the window
 }
@@ -685,7 +704,7 @@ void Send_message_call( /*@unused@*/ Widget w, XtPointer clientData, /*@unused@*
             sizeof(call),
             "%s",
             (char *)clientData);
-        Send_message(Global.top,call,NULL);
+        Send_message(appshell, call, NULL);
     }           
 }
 
@@ -755,8 +774,7 @@ begin_critical_section(&send_message_dialog_lock, "messages_gui.c:Send_message"
 
         mw[i].message_group = groupon;
         mw[i].send_message_dialog = XtVaCreatePopupShell(temp,
-                xmDialogShellWidgetClass,
-                Global.top,
+                xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 XmNtitleString,"Send Message",
@@ -1152,8 +1170,7 @@ begin_critical_section(&auto_msg_dialog_lock, "messages_gui.c:Auto_msg_set" );
     if(!auto_msg_dialog) {
 
         auto_msg_dialog = XtVaCreatePopupShell(langcode("WPUPARM001"),
-                xmDialogShellWidgetClass,
-                Global.top,
+                xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
