@@ -1,9 +1,9 @@
 /*
- * $Id: popup.h,v 1.3 2004/01/26 16:18:23 we7u Exp $
+ * $Id: popup.h,v 1.6 2006/01/17 21:05:20 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2004  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@ typedef struct
 /* from popup_gui.c */
 extern void popup_gui_init(void);
 extern void clear_popup_message_windows(void);
-extern void popup_time_out_check(void);
+extern void popup_time_out_check(int curr_sec);
 
 #endif /* XASTIR_POPUP_H */
 
