@@ -1,6 +1,6 @@
 
-// $Id: shp_hash.h,v 1.3 2005/01/08 09:46:48 we7u Exp $
-// Copyright (C) 2004-2005  The Xastir Group
+// $Id: shp_hash.h,v 1.5 2006/01/17 21:05:20 we7u Exp $
+// Copyright (C) 2004-2006  The Xastir Group
 
 
 #ifndef __XASTIR_SHP_HASH_H
@@ -29,10 +29,10 @@ typedef struct _shpinfo{
 void init_shp_hash(int clobber);
 void add_shp_to_hash(char *filename,SHPHandle sHP);
 void build_rtree(struct Node **root, SHPHandle sHP);
-void destroy_shp_hash();
+void destroy_shp_hash(void);
 void empty_shpinfo(shpinfo *si);
 void destroy_shpinfo(shpinfo *si);
-void purge_shp_hash();
+void purge_shp_hash(void);
 shpinfo *get_shp_from_hash(char *filename);
 
 #endif // USE_RTREE
