@@ -1,9 +1,9 @@
 /*
- * $Id: track_gui.c,v 1.39 2005/07/22 06:08:17 we7u Exp $
+ * $Id: track_gui.c,v 1.55 2006/04/24 19:21:42 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,10 @@
  * Look at the README for more information on the program.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "snprintf.h"
 
 #include <stdio.h>
@@ -35,10 +38,16 @@
 #include "xastir.h"
 #include "main.h"
 #include "lang.h"
+#include "objects.h"
 #include "popup.h"
 #include "util.h"
 #include "xa_config.h"
 
+// Must be last include file
+#include "leak_detection.h"
+
+
+
 Widget track_station_dialog = (Widget)NULL;
 Widget track_station_data = (Widget)NULL;
 Widget download_findu_dialog = (Widget)NULL;
@@ -54,6 +63,7 @@ Widget download_trail_station_data;
 Widget posit_start_value;
 Widget posit_length_value;
 
+int fetching_findu_trail_now = 0;
 
 int track_station_on;           /* used for tracking stations */
 int track_me;
@@ -62,8 +72,13 @@ int track_match;                /* used for tracking stations */
 char tracking_station_call[30]; /* Tracking station callsign */
 char download_trail_station_call[30];   /* Trail station callsign */
 //N0VH
-int posit_start = 336;
-int posit_length = 336;
+#define MAX_FINDU_DURATION 120
+#define MAX_FINDU_START_TIME 336
+// Make these two match, as that will be the most desired case: Snag
+// the track for as far back as possible up to the present in one
+// shot...
+int posit_start = MAX_FINDU_DURATION;
+int posit_length = MAX_FINDU_DURATION;
 
 
 
@@ -125,7 +140,6 @@ void Track_station_now(Widget w, XtPointer clientData, XtPointer callData) {
     char *temp_ptr;
 
 
-    /* find station and go there */
     temp_ptr = XmTextFieldGetString(track_station_data);
     xastir_snprintf(temp,
         sizeof(temp),
@@ -134,6 +148,8 @@ void Track_station_now(Widget w, XtPointer clientData, XtPointer callData) {
     XtFree(temp_ptr);
 
     (void)remove_trailing_spaces(temp);
+    (void)remove_trailing_dash_zero(temp);
+
     xastir_snprintf(tracking_station_call,
         sizeof(tracking_station_call),
         "%s",
@@ -150,6 +166,7 @@ void Track_station_now(Widget w, XtPointer clientData, XtPointer callData) {
             xastir_snprintf(temp2, sizeof(temp2), langcode("POPEM00026"), temp);
             popup_message_always(langcode("POPEM00025"),temp2);
         }
+        // Check for exact match, includes SSID
         if ( track_me & !is_my_call( tracking_station_call, 1) ) {
             XmToggleButtonSetState( trackme_button, FALSE, FALSE );
             track_me = 0;
@@ -178,9 +195,8 @@ void Track_station( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientData, /*
 begin_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" );
 
         track_station_dialog = XtVaCreatePopupShell(langcode("WPUPTSP001"),
-                xmDialogShellWidgetClass,
-                Global.top,
-                XmNdeleteResponse,XmDESTROY,
+                xmDialogShellWidgetClass, appshell,
+                XmNdeleteResponse, XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
 
@@ -380,6 +396,84 @@ end_critical_section(&track_station_dialog_lock, "track_gui.c:Track_station" );
 /***** DOWNLOAD FINDU TRAILS *****/
 
 
+// This is the separate execution thread that fetches the track from
+// findu.  The thread is started up by the Download_trail_now()
+// function below.
+//
+static void* findu_transfer_thread(void *arg) {
+    char *fileimg;
+    char *log_filename;
+    char **ptrs;
+
+
+    // Get fileimg and log_filename from parameters
+    ptrs = arg;
+    log_filename = ptrs[0];
+    fileimg = ptrs[1];
+
+    // Set global "busy" variable
+    fetching_findu_trail_now = 1;
+ 
+    if (fetch_remote_file(fileimg, log_filename)) {
+
+        // Had trouble getting the file.  Abort.
+
+// We may not be able to do any GUI stuff from multiple
+// threads/processes at the same time.  If that is found to be the
+// case we can write to STDERR instead.
+ 
+        // Dump a message to STDERR
+//        fprintf(stderr,
+//            "%s  %s\n",
+//            langcode("POPEM00035"),
+//            langcode("POPEM00044"));
+
+        // Fetch Findu Trail: Failed
+        popup_message_always(langcode("POPEM00035"),
+            langcode("POPEM00044"));
+
+        // Reset global "busy" variable
+        fetching_findu_trail_now = 0;
+        
+        return(NULL);
+    }
+
+// We need to move this message up to the main thread if possible so
+// that we don't have multiple threads writing to the GUI.  This
+// sort of operation can cause segfaults.  In practice I haven't
+// seen any segfaults due to this particular popup though.
+
+    // Fetch Findu Trail: Complete
+    popup_message_always(langcode("POPEM00036"),
+        langcode("POPEM00045"));
+
+    // Set permissions on the file so that any user can overwrite it.
+    chmod(log_filename, 0666);
+
+    // Here we do the findu stuff, if the findu_flag is set.  Else we do an imagemap.
+    // We have the log data we're interested in within the log_filename file.
+    // Cause that file to be read by the "File->Open Log File" routine.  HTML
+    // tags will be ignored just fine.
+    read_file_ptr = fopen(log_filename, "r");
+
+    if (read_file_ptr != NULL) {
+        read_file = 1;
+    }
+    else {
+        fprintf(stderr,"Couldn't open file: %s\n", log_filename);
+    }
+
+    // Reset global "busy" variable
+    fetching_findu_trail_now = 0;
+ 
+    // End the thread
+    return(NULL);
+}
+
+
+
+
+
 void Download_trail_destroy_shell( /*@unused@*/ Widget widget, XtPointer clientData, /*@unused@*/ XtPointer callData) {
     Widget shell = (Widget) clientData;
     XtPopdown(shell);
@@ -398,21 +492,37 @@ end_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_trail_de
 
 void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
     char temp[MAX_CALLSIGN+1];
-    char fileimg[400];
-    char log_filename[200];
+    static char fileimg[400];
+    static char log_filename[200];
     char *temp_ptr;
+    pthread_t download_trail_thread;
+    static XtPointer download_client_data = NULL;
+    static char *download_client_ptrs[2];
 
 
+    // If we're already fetching a trail, we shouldn't be calling
+    // this callback function.  Get out.
+    if (fetching_findu_trail_now)
+        return;
+
+    download_client_ptrs[0] = log_filename;
+    download_client_ptrs[1] = fileimg;
+    download_client_data = (XtPointer *)download_client_ptrs;
+
     // Check whether it's ok to do a download currently.
     if (read_file) {
         // No, we're already in the middle of reading in some file.
         // Skip trying to download yet another file.
+
+        fprintf(stderr,"Processing another file.  Wait a bit, then try again\n");
+
         popup_message_always(langcode("POPEM00035"),
             langcode("POPEM00041"));
+
         return;
     }
  
-    busy_cursor(appshell);
+//    busy_cursor(appshell);
 
     xastir_snprintf(log_filename,
         sizeof(log_filename),
@@ -427,7 +537,7 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
 
     XmScaleGetValue(posit_start_value , &posit_start);
     XmScaleGetValue(posit_length_value , &posit_length);
-    /* find station and go there */
+
     temp_ptr = XmTextFieldGetString(download_trail_station_data);
     xastir_snprintf(temp,
         sizeof(temp),
@@ -436,6 +546,7 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
     XtFree(temp_ptr);
 
     (void)remove_trailing_spaces(temp);
+    (void)remove_trailing_dash_zero(temp);
    
     xastir_snprintf(download_trail_station_call,
         sizeof(download_trail_station_call),
@@ -462,24 +573,23 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
 //        "http://www.findu.com/cgi-bin/rawposit.cgi?call=%s&start=%d&length=%d&time=1",// New
         download_trail_station_call,posit_start,posit_length);
 
-    if (fetch_remote_file(fileimg, log_filename)) {
-        // Had trouble getting the file.  Abort.
-        return;
+    if (debug_level & 1024) {
+        fprintf(stderr, "%s\n", fileimg);
     }
 
-    // Set permissions on the file so that any user can overwrite it.
-    chmod(log_filename, 0666);
 
-    // Here we do the findu stuff, if the findu_flag is set.  Else we do an imagemap.
-    // We have the log data we're interested in within the log_filename file.
-    // Cause that file to be read by the "File->Open Log File" routine.  HTML
-    // tags will be ignored just fine.
-    read_file_ptr = fopen(log_filename, "r");
+//----- Start New Thread -----
 
-    if (read_file_ptr != NULL)
-        read_file = 1;
-    else
-        fprintf(stderr,"Couldn't open file: %s\n", log_filename);
+    if (pthread_create(&download_trail_thread,
+            NULL,
+            findu_transfer_thread,
+            download_client_data)) {
+
+        fprintf(stderr,"Error creating findu transfer thread\n");
+    }
+    else {
+        // We're off and running with the new thread!
+    }
 
     display_zoom_status();
 
@@ -492,19 +602,28 @@ void Download_trail_now(Widget w, XtPointer clientData, XtPointer callData) {
 
 void Reset_posit_length_max(Widget w, XtPointer clientData, XtPointer callData) {
 
-    int temp;
-
-    XmScaleGetValue(posit_length_value, &temp);
+    XmScaleGetValue(posit_length_value, &posit_length);
     XmScaleGetValue(posit_start_value, &posit_start);
-    if (temp >= posit_start) {
+
+    // Check whether start hours is greater than max findu allows
+    // for duration
+    //
+    if (posit_start > MAX_FINDU_DURATION) {    // Set the duration slider to
+                                // findu's max duration hours
+
+        XtVaSetValues(posit_length_value,
+            XmNvalue, MAX_FINDU_DURATION,
+            NULL);
+        posit_length = MAX_FINDU_DURATION;
+    }
+    else {  // Not near the max, so set the duration slider to match
+            // the start hours
+
         XtVaSetValues(posit_length_value,
             XmNvalue, posit_start,
             NULL);
-        posit_length = temp;
+        posit_length = posit_start;
     }
-    XtVaSetValues(posit_length_value,
-        XmNmaximum, posit_start,
-        NULL);
 }
 
 
@@ -521,8 +640,7 @@ void Download_findu_trail( /*@unused@*/ Widget w, /*@unused@*/ XtPointer clientD
 begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_trail" );
 
         download_findu_dialog = XtVaCreatePopupShell(langcode("WPUPTSP007"),
-                xmDialogShellWidgetClass,
-                Global.top,
+                xmDialogShellWidgetClass, appshell,
                 XmNdeleteResponse,XmDESTROY,
                 XmNdefaultPosition, FALSE,
                 NULL);
@@ -598,7 +716,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNorientation, XmHORIZONTAL,
                 XmNborderWidth, 1,
                 XmNminimum, 1,
-                XmNmaximum, 336,
+                XmNmaximum, MAX_FINDU_START_TIME,
                 XmNshowValue, TRUE,
                 XmNvalue, posit_start,
                 XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP009"), 22,
@@ -625,7 +743,7 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 XmNorientation, XmHORIZONTAL,
                 XmNborderWidth, 1,
                 XmNminimum, 1,
-                XmNmaximum, 336,
+                XmNmaximum, MAX_FINDU_DURATION,
                 XmNshowValue, TRUE,
                 XmNvalue, posit_length,
                 XtVaTypedArg, XmNtitleString, XmRString, langcode("WPUPTSP010"), 19,
@@ -665,6 +783,8 @@ begin_critical_section(&download_findu_dialog_lock, "track_gui.c:Download_findu_
                 MY_FOREGROUND_COLOR,
                 MY_BACKGROUND_COLOR,
                 NULL);
+        if (fetching_findu_trail_now)
+            XtSetSensitive(button_ok, FALSE);
 
         button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00002"),
                 xmPushButtonGadgetClass, 
