@@ -1,9 +1,9 @@
 /*
- * $Id: util.h,v 1.37 2005/07/29 19:29:23 we7u Exp $
+ * $Id: util.h,v 1.46 2006/04/24 17:20:44 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
  * Copyright (C) 1999,2000  Frank Giannandrea
- * Copyright (C) 2000-2005  The Xastir Group
+ * Copyright (C) 2000-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,6 +34,15 @@
 #define MAX_WIDES 3
 
 
+
+extern int convert_from_xastir_coordinates ( float *f_longitude,
+                                      float *f_latitude,
+                                      long x,
+                                      long y );
+extern int convert_to_xastir_coordinates (unsigned long* x,
+                      unsigned long* y,
+                      float f_longitude,
+                      float f_latitude);
 extern char *get_tactical_from_hash(char *callsign);
 extern void destroy_tactical_hash(void);
 extern void xastir_debug(int my_debug_level, char *debug_string);
@@ -41,6 +50,7 @@ extern char *remove_all_spaces(char *data);
 extern char *remove_leading_spaces(char *data);
 extern char *remove_trailing_spaces(char *data);
 extern char *remove_trailing_asterisk(char *data);
+extern char *remove_trailing_dash_zero(char *data);
 extern int  position_amb_chars;
 extern void start_timer(void);
 extern void stop_timer(void);
@@ -64,6 +74,12 @@ extern char *compress_posit(const char *lat, const char group, const char *lon,
 extern int  position_defined(long lat, long lon, int strict);
 extern void convert_screen_to_xastir_coordinates(int x, int y, long *lat, long *lon);
 extern void convert_xastir_to_UTM_str(char *str, int str_len, long x, long y);
+extern void convert_xastir_to_MGRS_str_components(char *utmZone, int utmZone_len, 
+                                           char *EastingL, int EastingL_len, 
+                                           char *NorthingL, int NorthingL_len, 
+                                           unsigned int *int_utmEasting, unsigned int *int_utmNorthing, 
+                                           long x,  long y, 
+                                           int nice_format, char *space_string, int space_string_len);
 extern void convert_xastir_to_MGRS_str(char *str, int str_len, long x, long y, int nice_format);
 extern void convert_xastir_to_UTM(double *easting, double *northing, char *zone, int zone_len, long x, long y);
 extern void convert_UTM_to_xastir(double easting, double northing, char *zone, long *x, long *y);
@@ -80,13 +96,13 @@ extern double distance_from_my_station(char *call_sign, char *course_deg);
 
 extern char *convert_bearing_to_name(char *bearing, int opposite);
 
+extern void compute_DR_position(long x_long, long y_lat, double range, double course, long *x_long2, long *y_lat2);
+extern void compute_current_DR_position(DataRow *p_station, long *x_long, long *y_lat);
+
 extern int  filethere(char *fn);
 extern int  filecreate(char *fn);
 extern time_t file_time(char *fn);
 extern void log_data(char *file, char *line);
-extern void disown_object_item(char *call_sign,char *new_owner);
-extern void log_object_item(char *line, int disable_object, char *object_name);
-extern void reload_object_item(void);
 extern void log_tactical_call(char *call_sign, char *tactical_call_sign);
 extern void reload_tactical_calls(void);
 extern time_t sec_now(void);
@@ -95,8 +111,6 @@ extern char *get_time(char *time_here);
 extern void substr(char *dest, char *src, int size);
 extern int  valid_path(char *path);
 extern int  valid_call(char *call);
-extern int  valid_object(char *name);
-extern int  valid_item(char *name);
 extern int  valid_inet_name(char *name, char *info, char *origin, int origin_size);
 
 extern char echo_digis[6][9+1];
@@ -139,7 +153,6 @@ extern void set_dangerous( char *ptr );
 extern void clear_dangerous(void);
 
 
-
 #endif // __XASTIR_UTIL_H
 
 
