@@ -1,8 +1,8 @@
 /*
- * $Id: x_spider.c,v 1.36 2005/07/17 06:17:56 we7u Exp $
+ * $Id: x_spider.c,v 1.51 2006/04/24 19:21:42 we7u Exp $
  *
  * XASTIR, Amateur Station Tracking and Information Reporting
- * Copyright (C) 2003-2005  The Xastir Group
+ * Copyright (C) 2003-2006  The Xastir Group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -101,7 +101,10 @@
 // multiplexer, repeater.
 
 
-//#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif  // HAVE_CONFIG_H
+
 #include "x_spider.h"
 #include "snprintf.h"
 
@@ -117,7 +120,7 @@
 #include <signal.h>
 #include <termios.h>
 #include <pwd.h>
-#include <pthread.h>
+
 #include <termios.h>
 #include <setjmp.h>
 #include <sys/socket.h>
@@ -166,6 +169,10 @@
 #define SIGRET  void
 #endif  // SIGRET
 
+// Must be last include file
+#include "leak_detection.h"
+
+
 
 // Define this if you wish to use this as a standalone program
 // instead of as a function in another program.
@@ -178,7 +185,10 @@
 // because it causes other problems.
 extern short checkHash(char *theCall, short theHash);
 extern void get_timestamp(char *timestring);
+extern void split_string( char *data, char *cptr[], int max );
 
+// From database.h
+extern char my_callsign[];
 
 
 typedef struct _pipe_object {
@@ -193,10 +203,16 @@ typedef struct _pipe_object {
 pipe_object *pipe_head = NULL;
 //int master_fd = -1; // Start with an invalid value
 
-pipe_object *xastir_pipe = NULL;
+pipe_object *xastir_tcp_pipe = NULL;
+pipe_object *xastir_udp_pipe = NULL;
 
-int pipe_xastir_to_server = -1;
-int pipe_server_to_xastir = -1;
+// TCP server pipes to/from Xastir proper
+int pipe_xastir_to_tcp_server = -1;
+int pipe_tcp_server_to_xastir = -1;
+
+// UDP server pipes to/from Xastir proper
+int pipe_xastir_to_udp_server = -1; // (not currently used)
+int pipe_udp_server_to_xastir = -1;
 
 
 
@@ -354,6 +370,11 @@ void str_echo2(int sockfd, int pipe_from_parent, int pipe_to_parent) {
     }
 
 
+    //Send our callsign to spider clients as "#callsign" much like APRS-IS sends "# javaAPRS"
+    // # xastir 1.5.1 callsign:<mycall>
+    sprintf(line,"# Welcome to Xastir's server port, callsign: %s\r\n",my_callsign);
+    writen(sockfd,line,strlen(line));
+
     // Infinite loop
     for ( ; ; ) {
 
@@ -689,7 +710,7 @@ int pipe_check(char *client_address) {
 //fprintf(stderr,"Data available, sending to server\n");
 //fprintf(stderr,"\t%s\n",line);
 
-                if (writen(pipe_server_to_xastir, line, n) != n) {
+                if (writen(pipe_tcp_server_to_xastir, line, n) != n) {
                     fprintf(stderr, "pipe_check: Writen error2: %d\n", errno);
                 }
             }
@@ -701,7 +722,7 @@ int pipe_check(char *client_address) {
 
     // Check the pipe from Xastir's main thread to see if it is
     // sending us any data
-    n = readline(pipe_xastir_to_server, line, MAXLINE);
+    n = readline(pipe_xastir_to_tcp_server, line, MAXLINE);
     if (n == 0) {
         exit(0); // Connection terminated
     }
@@ -765,8 +786,10 @@ int pipe_check(char *client_address) {
 /* Globals */
 static char **Argv = ((void *)0);
 #ifdef __linux__
+#ifndef __LSB__
 extern char *__progname, *__progname_full;
-#endif
+#endif  // __LSB__
+#endif  // __linux__
 static char *LastArgv = ((void *)0);
 
 
@@ -800,10 +823,12 @@ void init_set_proc_title(int argc, char *argv[], char *envp[]) {
         LastArgv = envp[i] + strlen(envp[i]);
     }
 #ifdef __linux__
+#ifndef __LSB__
     // Pretty sure you don't need this either
     __progname = strdup("xastir");
     __progname_full = strdup(argv[0]);
-#endif
+#endif  // __LSB__
+#endif  // __linux__
 }
 
 
@@ -868,7 +893,7 @@ void set_proc_title(char *fmt,...) {
 #ifdef STANDALONE_PROGRAM
 int main(int argc, char *argv[]) {
 #else   // !STANDALONE_PROGRAM 
-void Server(int argc, char *argv[], char *envp[]) {
+void TCP_Server(int argc, char *argv[], char *envp[]) {
 #endif  // STANDALONE_PROGRAM
 
     int sockfd, newsockfd, childpid;
@@ -885,6 +910,7 @@ void Server(int argc, char *argv[], char *envp[]) {
     //
     if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
         fprintf(stderr,"x_spider: Can't open socket for listening\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
         exit(1);
     }
 
@@ -892,6 +918,7 @@ void Server(int argc, char *argv[], char *envp[]) {
     //
     if (fcntl(sockfd, F_SETFL, O_NONBLOCK) < 0) {
         fprintf(stderr,"x_spider: Couldn't set socket non-blocking\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
     }
 
     // Set up to reuse the port number (good for debug so we can
@@ -904,6 +931,7 @@ void Server(int argc, char *argv[], char *envp[]) {
             (char *)&sendbuff,
             sizeof(sendbuff)) < 0) {
         fprintf(stderr,"x_spider: Couldn't set socket REUSEADDR\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
     }
 
     // Bind our local address so that the client can send to us.
@@ -917,6 +945,7 @@ void Server(int argc, char *argv[], char *envp[]) {
             (struct sockaddr *)&serv_addr,
             sizeof(serv_addr)) < 0) {
         fprintf(stderr,"x_spider: Can't bind local address\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
         exit(1);
     }
 
@@ -969,6 +998,7 @@ void Server(int argc, char *argv[], char *envp[]) {
                 // of the loop.
                 //
                 fprintf(stderr,"x_spider: Accept error: %d\n", errno);
+                fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
                 goto finis;
             }
         }
@@ -1035,6 +1065,12 @@ void Server(int argc, char *argv[], char *envp[]) {
             // child process
             //
 
+
+            // Go back to default signal handler instead of calling
+            // restart() on SIGHUP
+            (void) signal(SIGHUP,SIG_DFL);
+
+
 /*
             fprintf(stderr,
                 "Client address: %s\n",
@@ -1097,6 +1133,7 @@ void Server(int argc, char *argv[], char *envp[]) {
         //
         if (fcntl(p->to_parent[0], F_SETFL, O_NONBLOCK) < 0) {
             fprintf(stderr,"x_spider: Couldn't set read-end of pipe_to_parent non-blocking\n");
+            fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
         }
 
 finis:
@@ -1113,6 +1150,256 @@ finis:
 
 
 
+// Send a nack back to the xastir_udp_client program 
+void send_udp_nack(int sock, struct sockaddr_in from, int fromlen) {
+    int n;
+
+    n = sendto(sock,
+        "NACK", // Negative Acknowledgment
+        5,
+        0,
+        (struct sockaddr *)&from,
+        fromlen);
+    if (n < 0) {
+        fprintf(stderr, "Error: sendto");
+    }
+}
+ 
+
+
+
+
+// Create a UDP listening port.  This allows scripts and other
+// programs to inject packets into Xastir via UDP protocol.
+//
+void UDP_Server(int argc, char *argv[], char *envp[]) {
+    int sock, length, n1, n2;
+    socklen_t fromlen;
+    struct sockaddr_in server;
+    struct sockaddr_in from;
+    char buf[1024];
+    char buf2[512];
+    char *callsign;
+    short passcode;
+    char *cptr[10];
+    char *message = NULL;
+    char message2[1024];
+    int send_to_inet;
+    int send_to_rf;
+
+
+    sock = socket(AF_INET, SOCK_DGRAM, 0);
+
+    if (sock < 0) {
+        fprintf(stderr, "Error: Opening socket");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
+        return;
+    }
+
+    length = sizeof(server);
+    memset(&server, 0, length);
+
+    server.sin_family = AF_INET;
+    server.sin_addr.s_addr = INADDR_ANY;
+    server.sin_port = htons(SERV_UDP_PORT);
+
+    if (bind(sock, (struct sockaddr *)&server, length) < 0) {
+        fprintf(stderr, "Error: Binding");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
+        return;
+    }
+
+    fromlen = sizeof(struct sockaddr_in);
+
+    while (1) {
+        n1 = recvfrom(sock,
+            buf,
+            1024,
+            0,
+            (struct sockaddr *)&from,
+            &fromlen);
+        if (n1 < 0) {
+            fprintf(stderr, "Error: recvfrom");
+        }
+        else if (n1 == 0) {
+            continue;
+        }
+        else {
+            buf[n1] = '\0';    // Terminate the buffer
+        }
+
+fprintf(stderr, "Received datagram: %s", buf);
+
+
+        send_to_inet = 0;
+        send_to_rf = 0;
+
+
+        //
+        // Authenticate the packet.  First line should contain:
+        //
+        //      callsign,passcode,[TO_RF],[TO_INET]
+        //
+        // The second line should contain the APRS packet
+        //
+
+        // Copy the entire buffer so that we can modify it
+        xastir_snprintf(buf2, sizeof(buf2), "%s", buf);
+        split_string(buf2, cptr, 10);
+
+        if (cptr[0] == NULL || cptr[0][0] == '\0') {    // callsign
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+        callsign = cptr[0];
+
+        if (cptr[1] == NULL || cptr[1][0] == '\0') {    // passcode
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+        passcode = atoi(cptr[1]);
+
+fprintf(stderr,"x_spider udp:  user:%s  pass:%d\n", callsign, passcode);
+
+        if (checkHash(callsign, passcode)) {
+            // Authenticate the pipe.  It is now allowed to send
+            // to the upstream server.
+            //fprintf(stderr,
+            //    "x_spider: Authenticated user %s\n",
+            //    callsign);
+        }
+        else {
+            fprintf(stderr,
+                "X_spider: Bad authentication, user %s, pass %d\n",
+                callsign,
+                passcode);
+            fprintf(stderr,
+                "UDP Packet: %s\n",
+                buf);
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+
+        // Here's where we would look for the optional flags in the
+        // first line.  Here we implement these flags:
+        //      -identify
+        //      -to_rf
+        //      -to_inet
+
+
+        // Look for the "-identify" flag in the UDP packet
+        //
+        if (strstr(buf, "-identify")) {
+            
+            // Send the callsign back to the xastir_udp_client
+            // program
+            n1 = sendto(sock,
+                my_callsign,
+                strlen(my_callsign)+1,
+                0,
+                (struct sockaddr *)&from,
+                fromlen);
+            if (n1 < 0) {
+                fprintf(stderr, "Error: sendto");
+            }
+            continue;
+        }
+
+
+        // Look for the "-to_inet" flag in the UDP packet
+        //
+        if (strstr(buf, "-to_inet")) {
+//fprintf(stderr,"Sending to INET\n");
+            send_to_inet++;
+        }
+
+
+        // Look for the "-to_rf" flag in the UDP packet
+        //
+        if (strstr(buf, "-to_rf")) {
+//fprintf(stderr,"Sending to local RF\n");
+            send_to_rf++;
+        }
+
+
+        // Now snag the text message from the second line using the
+        // original buffer.  Look for the first '\n' character which
+        // is just before the text message itself.
+        message = strchr(buf, '\n');
+        message++;  // Point to the first char after the '\n'
+
+        if (message == NULL || message[0] == '\0') {
+//fprintf(stderr,"Empty message field\n");
+            send_udp_nack(sock, from, fromlen);
+            continue;
+        }
+
+//fprintf(stderr,"Message:  %s", message);
+
+        xastir_snprintf(message2,
+            sizeof(message2),
+            "%s%s%s",
+            (send_to_inet) ? "TO_INET," : "",
+            (send_to_rf) ? "TO_RF," : "",
+            message);
+
+//fprintf(stderr,"Message2: %s", message2);
+
+
+
+//
+//
+// NOTE:
+// Should we refuse to send the message on if "callsign" and the
+// FROM callsign in the packet don't match?
+//
+// Should we change to third-party format if "my_callsign" and the
+// FROM callsign in the packet don't match?
+//
+// Require all three callsigns to match?
+//
+//
+
+
+
+        n1 = strlen(message);
+        n2 = strlen(message2);
+
+
+        // Send to Xastir udp pipe
+        //
+//fprintf(stderr,"Sending to Xastir itself\n");
+        if (writen(pipe_udp_server_to_xastir, message2, n2) != n2) {
+            fprintf(stderr,"UDP_Server: Writen error1: %d\n", errno);
+        }
+
+        // Send to the x_spider TCP server, so it can go to all
+        // connected TCP clients
+//fprintf(stderr,"Sending to TCP clients\n");
+        if (writen(pipe_xastir_to_tcp_server, message, n1) != n1) {
+            fprintf(stderr, "UDP_Server: Writen error2: %d\n", errno);
+        }
+ 
+        // Send an ACK back to the xastir_udp_client program 
+        n1 = sendto(sock,
+            "ACK",  // Acknowledgment.  Good UDP packet.
+            4,
+            0,
+            (struct sockaddr *)&from,
+            fromlen);
+        if (n1 < 0) {
+            fprintf(stderr, "Error: sendto");
+        }
+    }
+}
+
+
+
+
+
 // Function used to start a separate process for the server.  This
 // way the server can be running concurrently with the main part of
 // Xastir.
@@ -1124,36 +1411,36 @@ finis:
 // communicate between Xastir and the server.
 //
 #ifndef STANDALONE_PROGRAM
-int Fork_server(int argc, char *argv[], char *envp[]) {
+int Fork_TCP_server(int argc, char *argv[], char *envp[]) {
     int childpid;
 
 
     // Allocate a pipe before we fork.
     //
-    xastir_pipe = (pipe_object *)malloc(sizeof(pipe_object));
-    if (xastir_pipe == NULL) {
+    xastir_tcp_pipe = (pipe_object *)malloc(sizeof(pipe_object));
+    if (xastir_tcp_pipe == NULL) {
         fprintf(stderr,"x_spider: Couldn't malloc pipe_object\n");
         return(0);
     }
 
-    if (pipe(xastir_pipe->to_child) < 0 || pipe(xastir_pipe->to_parent) < 0) {
+    if (pipe(xastir_tcp_pipe->to_child) < 0 || pipe(xastir_tcp_pipe->to_parent) < 0) {
         fprintf(stderr,"x_spider: Can't create pipes\n");
-        free(xastir_pipe);    // Free the malloc'd memory.
+        free(xastir_tcp_pipe);    // Free the malloc'd memory.
         return(0);
     }
  
-    xastir_pipe->authenticated = 1;
-    xastir_pipe->callsign[0] = '\0';
+    xastir_tcp_pipe->authenticated = 1;
+    xastir_tcp_pipe->callsign[0] = '\0';
  
     if ( (childpid = fork()) < 0) {
-        fprintf(stderr,"Fork_server: Fork error\n");
+        fprintf(stderr,"Fork_TCP_server: Fork error\n");
 
         // Close pipes
-        close(xastir_pipe->to_child[0]);
-        close(xastir_pipe->to_child[1]);
-        close(xastir_pipe->to_parent[0]);
-        close(xastir_pipe->to_parent[1]);
-        free(xastir_pipe);    // Free the malloc'd memory.
+        close(xastir_tcp_pipe->to_child[0]);
+        close(xastir_tcp_pipe->to_child[1]);
+        close(xastir_tcp_pipe->to_parent[0]);
+        close(xastir_tcp_pipe->to_parent[1]);
+        free(xastir_tcp_pipe);    // Free the malloc'd memory.
         return(0);
     }
     else if (childpid == 0) {
@@ -1162,63 +1449,70 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
         //
 
 
+        // Go back to default signal handler instead of calling
+        // restart() on SIGHUP
+        (void) signal(SIGHUP,SIG_DFL);
+
+
         // Change the name of the new child process.  So far this
         // only works for "ps" listings, not for "top".  This code
         // only works on Linux.  For BSD use setproctitle(3), NetBSD
         // can use setprogname(2).
 #ifdef __linux__
         init_set_proc_title(argc, argv, envp);
-        set_proc_title("%s", "x-spider daemon (xastir)");
+        set_proc_title("%s", "x-spider TCP daemon (xastir)");
         //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
 #endif  // __linux__
  
 
-        close(xastir_pipe->to_child[1]);  // Close write end of pipe
-        close(xastir_pipe->to_parent[0]); // Close read end of pipe
+        close(xastir_tcp_pipe->to_child[1]);  // Close write end of pipe
+        close(xastir_tcp_pipe->to_parent[0]); // Close read end of pipe
 
         // Assign the global variables
-        pipe_server_to_xastir = xastir_pipe->to_parent[1];
-        pipe_xastir_to_server = xastir_pipe->to_child[0];
+        pipe_tcp_server_to_xastir = xastir_tcp_pipe->to_parent[1];
+        pipe_xastir_to_tcp_server = xastir_tcp_pipe->to_child[0];
 
         // Set read-end of pipe to be non-blocking.
         //
-        if (fcntl(pipe_xastir_to_server, F_SETFL, O_NONBLOCK) < 0) {
-            fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_server non-blocking\n");
+        if (fcntl(pipe_xastir_to_tcp_server, F_SETFL, O_NONBLOCK) < 0) {
+            fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_tcp_server non-blocking\n");
+            fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
         }
 
         // Go into an infinite loop here which restarts the
         // listening process whenever it dies.
         //
 //        while (1) {
-//            fprintf(stderr,"Starting Server...\n");
+//            fprintf(stderr,"Starting TCP_Server...\n");
 
-            Server(argc, argv, envp);
+            TCP_Server(argc, argv, envp);
  
-//            fprintf(stderr,"Server process died.\n");
+//            fprintf(stderr,"TCP_Server process died.\n");
 //        }
     }
     //
     // Parent process
     //
 
-    close(xastir_pipe->to_parent[1]); // Close write end of pipe
-    close(xastir_pipe->to_child[0]);  // Close read end of pipe
+    close(xastir_tcp_pipe->to_parent[1]); // Close write end of pipe
+    close(xastir_tcp_pipe->to_child[0]);  // Close read end of pipe
 
     // Assign the global variables so that Xastir itself will know
     // how to talk to the pipes
-    pipe_server_to_xastir = xastir_pipe->to_parent[0];
-    pipe_xastir_to_server = xastir_pipe->to_child[1];
+    pipe_tcp_server_to_xastir = xastir_tcp_pipe->to_parent[0];
+    pipe_xastir_to_tcp_server = xastir_tcp_pipe->to_child[1];
 
     // Set read-end of pipe to be non-blocking.
     //
-    if (fcntl(pipe_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
-        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_server_to_xastir non-blocking\n");
+    if (fcntl(pipe_tcp_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
+        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_tcp_server_to_xastir non-blocking\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
     }
 
 //    // Set write-end of pipe to be non-blocking.
 //    //
-//    if (fcntl(pipe_xastir_to_server, F_SETFL, O_NONBLOCK) < 0) {
-//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_server non-blocking\n");
+//    if (fcntl(pipe_xastir_to_tcp_server, F_SETFL, O_NONBLOCK) < 0) {
+//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_tcp_server non-blocking\n");
 //    }
 
     // We don't need to do anything here except return back to the
@@ -1230,3 +1524,120 @@ int Fork_server(int argc, char *argv[], char *envp[]) {
 #endif  // STANDALONE_PROGRAM
 
 
+
+
+
+int Fork_UDP_server(int argc, char *argv[], char *envp[]) {
+    int childpid;
+
+
+    // Allocate a pipe before we fork.
+    //
+    xastir_udp_pipe = (pipe_object *)malloc(sizeof(pipe_object));
+    if (xastir_udp_pipe == NULL) {
+        fprintf(stderr,"x_spider: Couldn't malloc pipe_object\n");
+        return(0);
+    }
+
+    if (pipe(xastir_udp_pipe->to_child) < 0 || pipe(xastir_udp_pipe->to_parent) < 0) {
+        fprintf(stderr,"x_spider: Can't create pipes\n");
+        free(xastir_udp_pipe);    // Free the malloc'd memory.
+        return(0);
+    }
+ 
+    xastir_udp_pipe->authenticated = 1;
+    xastir_udp_pipe->callsign[0] = '\0';
+ 
+    if ( (childpid = fork()) < 0) {
+        fprintf(stderr,"Fork_UDP_server: Fork error\n");
+
+        // Close pipes
+        close(xastir_udp_pipe->to_child[0]);
+        close(xastir_udp_pipe->to_child[1]);
+        close(xastir_udp_pipe->to_parent[0]);
+        close(xastir_udp_pipe->to_parent[1]);
+        free(xastir_udp_pipe);    // Free the malloc'd memory.
+        return(0);
+    }
+    else if (childpid == 0) {
+        //
+        // Child process
+        //
+
+
+        // Go back to default signal handler instead of calling
+        // restart() on SIGHUP
+        (void) signal(SIGHUP,SIG_DFL);
+
+
+        // Change the name of the new child process.  So far this
+        // only works for "ps" listings, not for "top".  This code
+        // only works on Linux.  For BSD use setproctitle(3), NetBSD
+        // can use setprogname(2).
+#ifdef __linux__
+        init_set_proc_title(argc, argv, envp);
+        set_proc_title("%s", "x-spider UDP daemon (xastir)");
+        //fprintf(stderr,"DEBUG: %s\n", Argv[0]);
+#endif  // __linux__
+ 
+
+        close(xastir_udp_pipe->to_child[1]);  // Close write end of pipe
+        close(xastir_udp_pipe->to_parent[0]); // Close read end of pipe
+
+        // Assign the global variables
+        pipe_udp_server_to_xastir = xastir_udp_pipe->to_parent[1];
+        pipe_xastir_to_udp_server = xastir_udp_pipe->to_child[0];
+
+        // Set read-end of pipe to be non-blocking.
+        //
+//        if (fcntl(pipe_xastir_to_udp_server, F_SETFL, O_NONBLOCK) < 0) {
+//            fprintf(stderr,
+//                "x_spider: Couldn't set read-end of pipe_xastir_to_udp_server non-blocking\n");
+//        }
+
+        // Go into an infinite loop here which restarts the
+        // listening process whenever it dies.
+        //
+//        while (1) {
+//            fprintf(stderr,"Starting UDP_Server...\n");
+
+            UDP_Server(argc, argv, envp);
+ 
+            fprintf(stderr,"UDP_Server process died.\n");
+//        }
+    }
+    //
+    // Parent process
+    //
+
+    close(xastir_udp_pipe->to_parent[1]); // Close write end of pipe
+    close(xastir_udp_pipe->to_child[0]);  // Close read end of pipe
+
+    // Assign the global variables so that Xastir itself will know
+    // how to talk to the pipes
+    pipe_udp_server_to_xastir = xastir_udp_pipe->to_parent[0];
+    pipe_xastir_to_udp_server = xastir_udp_pipe->to_child[1];
+
+
+    // Set read-end of pipe to be non-blocking.
+    //
+    if (fcntl(pipe_udp_server_to_xastir, F_SETFL, O_NONBLOCK) < 0) {
+        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_udp_server_to_xastir non-blocking\n");
+        fprintf(stderr,"Could some processes still be running from a previous run of Xastir?\n");
+    }
+
+//    // Set write-end of pipe to be non-blocking.
+//    //
+//    if (fcntl(pipe_xastir_to_udp_server, F_SETFL, O_NONBLOCK) < 0) {
+//        fprintf(stderr,"x_spider: Couldn't set read-end of pipe_xastir_to_udp_server non-blocking\n");
+//    }
+
+
+    // We don't need to do anything here except return back to the
+    // calling routine with the PID of the new server process, so
+    // that it can request the server and all it's children to quit
+    // when Xastir quits or segfaults.
+    return(childpid);   // Really the parent PID in this case
+}
+
+ 
